#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/util.h>
#include <inttypes.h>
#include <nrfx_pwm.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/logging/log.h>
#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>
#include <bluetooth/services/lbs.h>
#include <debug/cpu_load.h>
#include "user_sensor_info.h"
#include "user_task_segger_input.h"


#if defined(CONFIG_RGB_LED_DRIVER) && CONFIG_RGB_LED_DRIVER
#include "user_rgb_led.h"
#endif

#include <stdio.h>

LOG_MODULE_REGISTER(main, LOG_LEVEL_INF);
static uint8_t log_text[200];

#define SLEEP_TIME_MS 1
#define PWM_INST_IDX 0
#define SW0_NODE DT_ALIAS(sw0)

#if !DT_NODE_HAS_STATUS(SW0_NODE, okay)
#error "Unsupported board: sw0 devicetree alias is not defined"
#endif

#define DEVICE_NAME CONFIG_BT_DEVICE_NAME
#define DEVICE_NAME_LEN (sizeof(DEVICE_NAME) - 1)

sensor_info_t sensor_info;

static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET_OR(SW0_NODE, gpios, {0});
static struct gpio_callback button_cb_data;

static int err;

static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};

static const struct bt_data sd[] = {
	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BT_UUID_LBS_VAL),
};

static void connected(struct bt_conn *conn, uint8_t err)
{
	if (err)
	{
		LOG_ERR("Connection failed (err %u)\n", err);
		return;
	}
	LOG_INF("Connected\n");
}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	LOG_INF("Disconnected (reason %u)\n", reason);
}

BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected = connected,
	.disconnected = disconnected,
#ifdef CONFIG_BT_LBS_SECURITY_ENABLED
	.security_changed = security_changed,
#endif
};

void button_pressed(const struct device *dev, struct gpio_callback *cb,
					uint32_t pins)
{
	// led_seq_r.repeats = PWM_REPEAT_RUSH;
	//	nrfx_pwm_simple_playback(&pwm_instance, &led_seq_r, 3, NRFX_PWM_FLAG_STOP);

	LOG_INF("trip blink");
}

static void timer0_handler(struct k_timer *dummy)
{
	/*Interrupt Context - System Timer ISR */
	static bool flip = true;
	// if (flip)
	// gpio_pin_toggle_dt(&led);
	// else
	// gpio_pin_toggle_dt(&led);

	flip = !flip;
	LOG_INF("timer0 cb");
}

K_TIMER_DEFINE(timer0, timer0_handler, NULL);

int main(void)
{
	// nrfx_err_t status;
	int ret;

	sensor_info.sync_addr[0] = 0xC7;
	sensor_info.sync_addr[1] = 0x94;
	sensor_info.sync_addr[2] = 0x62;
	sensor_info.sync_addr[3] = 0x4E;
	sensor_info.sync_addr[4] = 0x01;
	sensor_info.sync_ch = 0;
	sensor_info.sync_mode = SYNC_TYPE_SLAVE;

	cpu_load_init();
	if (!gpio_is_ready_dt(&button))
	{
		LOG_ERR("Error: button device %s is not ready",
				button.port->name);
		return 0;
	}

	ret = gpio_pin_configure_dt(&button, GPIO_INPUT);
	if (ret != 0)
	{
		LOG_ERR("Error %d: failed to configure %s pin %d",
				ret, button.port->name, button.pin);
		return 0;
	}

	ret = gpio_pin_interrupt_configure_dt(&button,
										  GPIO_INT_EDGE_TO_ACTIVE);
	if (ret != 0)
	{
		LOG_ERR("Error %d: failed to configure interrupt on %s pin %d",
				ret, button.port->name, button.pin);
		return 0;
	}

	// k_timer_start(&timer0, K_MSEC(500), K_MSEC(500));

	gpio_init_callback(&button_cb_data, button_pressed, BIT(button.pin));
	gpio_add_callback(button.port, &button_cb_data);
	LOG_INF("Set up button at %s pin %d", button.port->name, button.pin);

	err = bt_enable(NULL);
	if (err)
	{
		LOG_ERR("Bluetooth init failed (err %d)\n", err);
		return 0;
	}

	LOG_INF("Bluetooth initialized\n");

	err = bt_lbs_init(NULL);
	if (err)
	{
		LOG_ERR("Failed to init LBS (err:%d)\n", err);
		return 0;
	}

	err = bt_le_adv_start(BT_LE_ADV_CONN, ad, ARRAY_SIZE(ad),
						  sd, ARRAY_SIZE(sd));
	if (err)
	{
		LOG_ERR("Advertising failed to start (err %d)\n", err);
		return 0;
	}

	LOG_INF("Advertising successfully started\n");

	return 0;
}

K_THREAD_DEFINE(user_task_segger_input_id, 4096, user_task_segger_input, (void *)&sensor_info, NULL, NULL, 7, 0, 0);